@@ -2,7 +2,7 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
+# Copyright (c) 2009 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of SIP.
 # 
@@ -15,9 +15,8 @@
 
 import sys
 import os
-import string
-import types
 import stat
+import string
 import re
 
 
@@ -26,7 +25,6 @@ _pkg_config = {
     'default_bin_dir':  '/usr/local/bin',
     'default_mod_dir':  '/usr/local/lib/python2.5/site-packages',
     'default_sip_dir':  '/usr/local/share/sip',
-    'export_all':       0,
     'platform':         'openbsd-g++',
     'py_conf_inc_dir':  '/usr/local/include/python2.5',
     'py_inc_dir':       '/usr/local/include/python2.5',
@@ -36,9 +34,9 @@ _pkg_config = {
     'sip_config_args':  '-b /usr/local/bin -d /usr/local/lib/python2.5/site-packages -e /usr/local/include/python2.5 -v /usr/local/share/sip INCDIR=/usr/local/include LIBDIR=/usr/local/lib CC=cc CXX=c++ CFLAGS=-O2 -pipe  CXXFLAGS=-O2 -pipe ',
     'sip_inc_dir':      '/usr/local/include/python2.5',
     'sip_mod_dir':      '/usr/local/lib/python2.5/site-packages',
-    'sip_version':      0x040709,
-    'sip_version_str':  '4.7.9',
-    'universal':        None
+    'sip_version':      0x040802,
+    'sip_version_str':  '4.8.2',
+    'universal':        ''
 }
 
 _default_macros = {
@@ -179,7 +177,7 @@ class Configuration(object):
             except KeyError:
                 pass
 
-        raise AttributeError, "\"%s\" is not a valid configuration value or user option" % name
+        raise AttributeError("\"%s\" is not a valid configuration value or user option" % name)
 
     def build_macros(self):
         """Return the dictionary of platform specific build macros.
@@ -261,7 +259,7 @@ class _Macro:
         if isinstance(value, _UniqueList):
             value = value.as_list()
 
-        if type(value) == types.ListType:
+        if type(value) == list:
             self.extend(value)
         else:
             self.append(value)
@@ -338,7 +336,7 @@ class Makefile:
             # configuration).  Also allow a list of Qt v4 modules to be
             # specified.
             if configuration.qt_version >= 0x040000:
-                if type(qt) != types.ListType:
+                if type(qt) != list:
                     qt = ["QtCore", "QtGui"]
 
             self._threaded = configuration.qt_threaded
@@ -365,7 +363,7 @@ class Makefile:
         # Copy the macros and convert them all to instance lists.
         macros = configuration.build_macros()
 
-        for m in macros.keys():
+        for m in list(macros.keys()):
             # Allow the user to override the default.
             try:
                 val = getattr(configuration, m)
@@ -376,12 +374,12 @@ class Makefile:
             # space separated values rather than a single value that might
             # contain spaces.
             if m in ("DEFINES", "CONFIG") or m[:6] in ("INCDIR", "LIBDIR"):
-                val = string.split(val)
+                val = val.split()
 
             # We also want to treat lists of libraries in the same way so that
             # duplicates get eliminated.
             if m[:4] == "LIBS":
-                val = string.split(val)
+                val = val.split()
 
             self.__dict__[m] = _Macro(m, val)
 
@@ -427,7 +425,7 @@ class Makefile:
         # time and nobody complained.  For the moment we'll leave the code in
         # but it will never be used.
         if self._qt:
-            wcfg = string.split(self.config.qt_winconfig)
+            wcfg = self.config.qt_winconfig.split()
             win_shared = ("shared" in wcfg)
             win_exceptions = ("exceptions" in wcfg)
             win_rtti = ("rtti" in wcfg)
@@ -628,6 +626,8 @@ class Makefile:
                         defines.append("QT_OPENGL_LIB")
                     elif mod == "QtScript":
                         defines.append("QT_SCRIPT_LIB")
+                    elif mod == "QtScriptTools":
+                        defines.append("QT_SCRIPTTOOLS_LIB")
                     elif mod == "QtSql":
                         defines.append("QT_SQL_LIB")
                     elif mod == "QtTest":
@@ -666,12 +666,13 @@ class Makefile:
                     "QtNetwork":        ("QtCore", ),
                     "QtOpenGL":         ("QtCore", "QtGui"),
                     "QtScript":         ("QtCore", ),
+                    "QtScriptTools":    ("QtCore", "QtGui", "QtScript"),
                     "QtSql":            ("QtCore", ),
                     "QtSvg":            ("QtCore", "QtGui", "QtXml"),
                     "QtTest":           ("QtCore", "QtGui"),
                     "QtWebKit":         ("QtCore", "QtGui", "QtNetwork"),
                     "QtXml":            ("QtCore", ),
-                    "QtXmlPatterns":    ("QtCore", ),
+                    "QtXmlPatterns":    ("QtCore", "QtNetwork"),
                     "phonon":           ("QtCore", "QtGui"),
                     "QtDesigner":       ("QtCore", "QtGui"),
                     "QAxContainer":     ("QtCore", "QtGui")
@@ -706,17 +707,17 @@ class Makefile:
                         # be anywhere.
                         deps = _UniqueList()
 
-                        if mod in wdepmap.keys():
+                        if mod in list(wdepmap.keys()):
                             deps.extend(self.optional_list(wdepmap[mod]))
 
-                        if mod in qdepmap.keys():
+                        if mod in list(qdepmap.keys()):
                             for qdep in qdepmap[mod]:
                                 # Ignore the dependency if it is explicitly
                                 # linked.
                                 if qdep not in self._qt:
                                     libs.append(self.platform_lib(self._qt4_module_to_lib(qdep)))
 
-                                    if qdep in wdepmap.keys():
+                                    if qdep in list(wdepmap.keys()):
                                         deps.extend(self.optional_list(wdepmap[qdep]))
 
                         libs.extend(deps.as_list())
@@ -727,7 +728,7 @@ class Makefile:
                 qt_lib = self.config.qt_lib
 
                 if self.generator in ("MSVC", "MSVC.NET", "BMAKE") and win_shared:
-                    qt_lib = qt_lib + string.replace(version_to_string(self.config.qt_version), ".", "")
+                    qt_lib = qt_lib + version_to_string(self.config.qt_version).replace(".", "")
 
                     if self.config.qt_edition == "non-commercial":
                         qt_lib = qt_lib + "nc"
@@ -835,10 +836,10 @@ class Makefile:
             elif self.config.qt_version < 0x040200 or sys.platform == "darwin":
                 lib = lib + "_debug"
 
-        if sys.platform == "win32" and "shared" in string.split(self.config.qt_winconfig):
+        if sys.platform == "win32" and "shared" in self.config.qt_winconfig.split():
             if (mname in ("QtCore", "QtDesigner", "QtGui", "QtHelp",
-                          "QtNetwork", "QtOpenGL", "QtScript", "QtSql",
-                          "QtSvg", "QtTest", "QtWebKit", "QtXml",
+                          "QtNetwork", "QtOpenGL", "QtScript", "QtScriptTools",
+                          "QtSql", "QtSvg", "QtTest", "QtWebKit", "QtXml",
                           "QtXmlPatterns", "phonon") or
                 (self.config.qt_version >= 0x040200 and mname == "QtAssistant")):
                 lib = lib + "4"
@@ -858,7 +859,7 @@ class Makefile:
         name is the name of the macro.
         default is the default value
         """
-        s = string.join(self.optional_list(name))
+        s = ' '.join(self.optional_list(name))
 
         if not s:
             s = default
@@ -873,7 +874,7 @@ class Makefile:
         s = self.optional_string(name)
 
         if not s:
-            raise ValueError, "\"%s\" must have a non-empty value" % name
+            raise ValueError("\"%s\" must have a non-empty value" % name)
 
         return s
 
@@ -925,16 +926,16 @@ class Makefile:
         if os.access(prl_name, os.F_OK):
             try:
                 f = open(prl_name, "r")
-            except IOError, detail:
-                error("Unable to open \"%s\": %s" % (prl_name, detail))
+            except IOError:
+                error("Unable to open \"%s\"" % prl_name)
 
             line = f.readline()
             while line:
-                line = string.strip(line)
+                line = line.strip()
                 if line and line[0] != "#":
-                    eq = string.find(line, "=")
-                    if eq > 0 and string.strip(line[:eq]) == "QMAKE_PRL_LIBS":
-                        prl_libs = string.split(line[eq + 1:])
+                    eq = line.find("=")
+                    if eq > 0 and line[:eq].strip() == "QMAKE_PRL_LIBS":
+                        prl_libs = line[eq + 1:].split()
                         break
 
                 line = f.readline()
@@ -951,35 +952,35 @@ class Makefile:
         filename is the name of the build file.  If it is a dictionary instead
         then its contents are validated.
         """
-        if type(filename) is types.DictType:
+        if type(filename) == dict:
             bfname = "dictionary"
-            dict = filename
+            bdict = filename
         else:
             if self._dir:
                 bfname = os.path.join(self._dir, filename)
             else:
                 bfname = filename
 
-            dict = {}
+            bdict = {}
 
             try:
                 f = open(bfname, "r")
-            except IOError, detail:
-                error("Unable to open \"%s\": %s" % (bfname, detail))
+            except IOError:
+                error("Unable to open \"%s\"" % bfname)
 
             line_nr = 1
             line = f.readline()
 
             while line:
-                line = string.strip(line)
+                line = line.strip()
 
                 if line and line[0] != "#":
-                    eq = string.find(line, "=")
+                    eq = line.find("=")
 
                     if eq <= 0:
                         error("\"%s\" line %d: Line must be in the form 'name = value value...'." % (bfname, line_nr))
 
-                    dict[string.strip(line[:eq])] = string.strip(line[eq + 1:])
+                    bdict[line[:eq].strip()] = line[eq + 1:].strip()
 
                 line_nr = line_nr + 1
                 line = f.readline()
@@ -989,16 +990,16 @@ class Makefile:
         # Check the compulsory values.
         for i in ("target", "sources"):
             try:
-                dict[i]
+                bdict[i]
             except KeyError:
                 error("\"%s\" is missing from \"%s\"." % (i, bfname))
 
         # Get the optional values.
         for i in ("headers", "moc_headers"):
             try:
-                dict[i]
+                bdict[i]
             except KeyError:
-                dict[i] = ""
+                bdict[i] = ""
 
         # Generate the list of objects.
         if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
@@ -1008,20 +1009,20 @@ class Makefile:
 
         olist = []
 
-        for f in string.split(dict["sources"]):
+        for f in bdict["sources"].split():
             root, discard = os.path.splitext(f)
             olist.append(root + ext)
 
-        for f in string.split(dict["moc_headers"]):
+        for f in bdict["moc_headers"].split():
             if not self._qt:
                 error("\"%s\" defines \"moc_headers\" for a non-Qt module." % bfname)
 
             root, discard = os.path.splitext(f)
             olist.append("moc_" + root + ext)
 
-        dict["objects"] = string.join(olist)
+        bdict["objects"] = ' '.join(olist)
 
-        return dict
+        return bdict
 
     def clean_build_file_objects(self, mfile, build):
         """Generate the clean target.
@@ -1031,10 +1032,10 @@ class Makefile:
         """
         mfile.write("\t-%s $(TARGET)\n" % self.rm)
 
-        for f in string.split(build["objects"]):
+        for f in build["objects"].split():
             mfile.write("\t-%s %s\n" % (self.rm, f))
 
-        for f in string.split(build["moc_headers"]):
+        for f in build["moc_headers"].split():
             root, discard = os.path.splitext(f)
             mfile.write("\t-%s moc_%s.cpp\n" % (self.rm, root))
 
@@ -1056,15 +1057,15 @@ class Makefile:
 
         try:
             mfile = open(mfname, "w")
-        except IOError, detail:
-            error("Unable to create \"%s\": %s" % (mfname, detail))
+        except IOError:
+            error("Unable to create \"%s\"" % mfname)
 
         self.generate_macros_and_rules(mfile)
         self.generate_target_default(mfile)
         self.generate_target_install(mfile)
 
         if self._installs:
-            if type(self._installs) != types.ListType:
+            if type(self._installs) != list:
                 self._installs = [self._installs]
 
             for src, dst in self._installs:
@@ -1085,6 +1086,9 @@ class Makefile:
 
         cppflags = []
 
+        if not self._debug:
+            cppflags.append("-DNDEBUG")
+
         for f in self.optional_list("DEFINES"):
             cppflags.append("-D" + f)
 
@@ -1108,13 +1112,13 @@ class Makefile:
 
         libs.extend(self.optional_list("LIBS"))
 
-        mfile.write("CPPFLAGS = %s\n" % string.join(cppflags))
+        mfile.write("CPPFLAGS = %s\n" % ' '.join(cppflags))
 
         mfile.write("CFLAGS = %s\n" % self.optional_string("CFLAGS"))
         mfile.write("CXXFLAGS = %s\n" % self.optional_string("CXXFLAGS"))
         mfile.write("LFLAGS = %s\n" % self.optional_string("LFLAGS"))
 
-        mfile.write("LIBS = %s\n" % string.join(libs))
+        mfile.write("LIBS = %s\n" % ' '.join(libs))
 
         if self._qt:
             mfile.write("MOC = %s\n" % _quote(self.required_string("MOC")))
@@ -1230,7 +1234,7 @@ class Makefile:
 
         mfile.write("%s %s\n" % (self.mkdir, _quote(dst)))
 
-        if type(src) != types.ListType:
+        if type(src) != list:
             src = [src]
 
         # Get the strip command if needed.
@@ -1356,32 +1360,23 @@ class PythonModuleMakefile(Makefile):
         """
         Makefile.generate_target_install(self, mfile)
 
-        os.path.walk(self._moddir, self._visit, mfile)
-
-    def _visit(self, mfile, dirname, names):
-        """Install the files from a particular directory.
-
-        mfile is the file object.
-        dirname is the sub-directory.
-        names is the list of files to install from the sub-directory.
-        """
-        tail = dirname[len(self._moddir):]
-
-        flist = []
-        for f in list(names):
-            # Ignore certain files.
-            if f in ("Makefile", ):
-                continue
-
+        for root, dirs, files in os.walk(self._moddir):
             # Do not recurse into certain directories.
-            if f in (".svn", "CVS"):
-                names.remove(f)
-                continue
+            for skip in (".svn", "CVS"):
+                if skip in dirs:
+                    dirs.remove(skip)
 
-            if os.path.isfile(os.path.join(dirname, f)):
-                flist.append(os.path.join(self._srcdir + tail, f))
+            tail = root[len(self._moddir):]
+            flist = []
 
-        self.install_file(mfile, flist, self._dstdir + tail)
+            for f in files:
+                if f == "Makefile":
+                    continue
+
+                if os.path.isfile(os.path.join(root, f)):
+                    flist.append(os.path.join(self._srcdir + tail, f))
+
+            self.install_file(mfile, flist, self._dstdir + tail)
 
 
 class ModuleMakefile(Makefile):
@@ -1420,16 +1415,17 @@ class ModuleMakefile(Makefile):
             self._limit_exports = 0
         else:
             self._strip = strip
-
-            # The deprecated configuration flag has precedence.
-            if self.config.export_all:
-                self._limit_exports = 0
-            else:
-                self._limit_exports = not export_all
+            self._limit_exports = not export_all
 
         # Save the target name for later.
         self._target = self._build["target"]
 
+        # The name of the module entry point is Python version specific.
+        if self.config.py_version >= 0x030000:
+            self._entry_point = "PyInit_%s" % self._target
+        else:
+            self._entry_point = "init%s" % self._target
+
         if sys.platform != "win32" and static:
             self._target = "lib" + self._target
 
@@ -1479,17 +1475,11 @@ class ModuleMakefile(Makefile):
         self.LFLAGS.extend(self.optional_list(lflags_console))
 
         if sys.platform == "darwin":
-            # We use the -F flag to explictly specify the directory containing
-            # the Python framework rather than rely on the default search path.
-            # This allows Apple's Python to be used even if a later python.org
-            # version is also installed.
-            dl = string.split(sys.exec_prefix, os.sep)
-            try:
-                dl = dl[:dl.index("Python.framework")]
-            except ValueError:
+            dl = sys.exec_prefix.split(os.sep)
+            if "Python.framework" not in dl:
                 error("SIP requires Python to be built as a framework")
-            self.LFLAGS.append("-F%s" % string.join(dl, os.sep))
-            self.LFLAGS.append("-framework Python")
+
+            self.LFLAGS.append("-undefined dynamic_lookup")
 
         Makefile.finalise(self)
 
@@ -1530,10 +1520,10 @@ class ModuleMakefile(Makefile):
                         else:
                             self.LFLAGS.extend(['-z' 'noversion', '-M', '%s.exp' % self._target])
                     elif sys.platform[:5] == 'hp-ux':
-                        self.LFLAGS.extend(['-Wl,+e,init%s' % self._target])
+                        self.LFLAGS.extend(['-Wl,+e,%s' % self._entry_point])
                     elif sys.platform[:5] == 'irix' and self.required_string('LINK') != 'g++':
                         # Doesn't work when g++ is used for linking on IRIX.
-                        self.LFLAGS.extend(['-Wl,-exported_symbol,init%s' % self._target])
+                        self.LFLAGS.extend(['-Wl,-exported_symbol,%s' % self._entry_point])
 
                 # Force the shared linker if there is one.
                 link_shlib = self.optional_list("LINK_SHLIB")
@@ -1552,7 +1542,7 @@ class ModuleMakefile(Makefile):
 
         mname is the name of the module.
         """
-        raise ValueError, "module_as_lib() can only be used with SIP v3.x"
+        raise ValueError("module_as_lib() can only be used with SIP v3.x")
 
     def generate_macros_and_rules(self, mfile):
         """Generate the macros and rules generation.
@@ -1602,6 +1592,18 @@ class ModuleMakefile(Makefile):
 
         mfile is the file object.
         """
+        # Do these first so that it's safe for a sub-class to append additional
+        # commands to the real target, but make sure the default is correct.
+        mfile.write("\nall: $(TARGET)\n")
+        mfile.write("\n$(OFILES): $(HFILES)\n")
+
+        for mf in self._build["moc_headers"].split():
+            root, discard = os.path.splitext(mf)
+            cpp = "moc_" + root + ".cpp"
+
+            mfile.write("\n%s: %s\n" % (cpp, mf))
+            mfile.write("\t$(MOC) -o %s %s\n" % (cpp, mf))
+
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
         if self.generator in ("MSVC", "MSVC.NET"):
@@ -1621,7 +1623,7 @@ class ModuleMakefile(Makefile):
                 mfile.write("\t-%s $(TARGET)\n" % (self.rm))
                 mfile.write("\t$(LIB) $(TARGET) @&&|\n")
 
-                for of in string.split(self._build["objects"]):
+                for of in self._build["objects"].split():
                     mfile.write("+%s \\\n" % (of))
 
                 mfile.write("|\n")
@@ -1638,11 +1640,11 @@ class ModuleMakefile(Makefile):
 
                 try:
                     dfile = open(defname, "w")
-                except IOError, detail:
-                    error("Unable to create \"%s\": %s" % (defname, detail))
+                except IOError:
+                    error("Unable to create \"%s\"" % defname)
 
                 dfile.write("EXPORTS\n")
-                dfile.write("init%s=_init%s\n" % (self._target, self._target))
+                dfile.write("%s=_%s\n" % (self._entry_point, self._entry_point))
 
                 dfile.close()
 
@@ -1657,24 +1659,15 @@ class ModuleMakefile(Makefile):
                 if self._limit_exports:
                     # Create an export file for AIX, Linux and Solaris.
                     if sys.platform[:5] == 'linux':
-                        mfile.write("\t@echo '{ global: init%s; local: *; };' > %s.exp\n" % (self._target, self._target))
+                        mfile.write("\t@echo '{ global: %s; local: *; };' > %s.exp\n" % (self._entry_point, self._target))
                     elif sys.platform[:5] == 'sunos':
-                        mfile.write("\t@echo '{ global: init%s; local: *; };' > %s.exp\n" % (self._target, self._target))
+                        mfile.write("\t@echo '{ global: %s; local: *; };' > %s.exp\n" % (self._entry_point, self._target))
                     elif sys.platform[:3] == 'aix':
                         mfile.write("\t@echo '#!' >%s.exp" % self._target)
-                        mfile.write("; \\\n\t echo 'init%s' >>%s.exp\n" % (self._target, self._target))
+                        mfile.write("; \\\n\t echo '%s' >>%s.exp\n" % (self._entry_point, self._target))
 
                 mfile.write("\t$(LINK) $(LFLAGS) -o $(TARGET) $(OFILES) $(LIBS)\n")
 
-        mfile.write("\n$(OFILES): $(HFILES)\n")
-
-        for mf in string.split(self._build["moc_headers"]):
-            root, discard = os.path.splitext(mf)
-            cpp = "moc_" + root + ".cpp"
-
-            mfile.write("\n%s: %s\n" % (cpp, mf))
-            mfile.write("\t$(MOC) -o %s %s\n" % (cpp, mf))
-
     def generate_target_install(self, mfile):
         """Generate the install target.
 
@@ -1799,7 +1792,7 @@ class ProgramMakefile(Makefile):
             blflags = []
 
             for lf in lflags:
-                for f in string.split(lf):
+                for f in lf.split():
                     # Tell the compiler to pass the flags to the linker.
                     if f[-1] == "-":
                         f = "-l-" + f[1:-1]
@@ -1809,7 +1802,7 @@ class ProgramMakefile(Makefile):
                     # Remove any explicit object files otherwise the compiler
                     # will complain that they can't be found, but they don't
                     # seem to be needed.
-                    if string.lower(f[-4:]) != ".obj":
+                    if f[-4:].lower() != ".obj":
                         blflags.append(f)
 
             lflags = blflags
@@ -1821,7 +1814,7 @@ class ProgramMakefile(Makefile):
         if self.generator == "BMAKE":
             build.append(source)
 
-        return (exe, string.join(build))
+        return (exe, ' '.join(build))
 
     def finalise(self):
         """Finalise the macros for a program Makefile.
@@ -1847,7 +1840,7 @@ class ProgramMakefile(Makefile):
         mfile is the file object.
         """
         if not self._build:
-            raise ValueError, "pass a filename as build_file when generating a Makefile"
+            raise ValueError("pass a filename as build_file when generating a Makefile")
 
         target = self._build["target"]
 
@@ -1866,6 +1859,18 @@ class ProgramMakefile(Makefile):
 
         mfile is the file object.
         """
+        # Do these first so that it's safe for a sub-class to append additional
+        # commands to the real target, but make sure the default is correct.
+        mfile.write("\nall: $(TARGET)\n")
+        mfile.write("\n$(OFILES): $(HFILES)\n")
+
+        for mf in self._build["moc_headers"].split():
+            root, discard = os.path.splitext(mf)
+            cpp = "moc_" + root + ".cpp"
+
+            mfile.write("\n%s: %s\n" % (cpp, mf))
+            mfile.write("\t$(MOC) -o %s %s\n" % (cpp, mf))
+
         mfile.write("\n$(TARGET): $(OFILES)\n")
 
         if self.generator in ("MSVC", "MSVC.NET"):
@@ -1882,15 +1887,6 @@ class ProgramMakefile(Makefile):
         if self._manifest:
             mfile.write("\tmt -nologo -manifest $(TARGET).manifest -outputresource:$(TARGET);1\n")
 
-        mfile.write("\n$(OFILES): $(HFILES)\n")
-
-        for mf in string.split(self._build["moc_headers"]):
-            root, discard = os.path.splitext(mf)
-            cpp = "moc_" + root + ".cpp"
-
-            mfile.write("\n%s: %s\n" % (cpp, mf))
-            mfile.write("\t$(MOC) -o %s %s\n" % (cpp, mf))
-
     def generate_target_install(self, mfile):
         """Generate the install target.
 
@@ -1919,7 +1915,7 @@ def _quote(s):
 
     s is the string.
     """
-    if string.find(s, " ") >= 0:
+    if s.find(" ") >= 0:
         s = '"' + s + '"'
 
     return s
@@ -1956,13 +1952,13 @@ def read_version(filename, description, numdefine=None, strdefine=None):
     l = f.readline()
 
     while l and (need_num or need_str):
-        wl = string.split(l)
+        wl = l.split()
         if len(wl) >= 3 and wl[0] == "#define":
             if need_num and wl[1] == numdefine:
                 v = wl[2]
 
                 if v[0:2] == "0x":
-                    vers = string.atoi(v,16)
+                    vers = int(v, 16)
                 else:
                     dec = int(v)
                     maj = dec / 100
@@ -1974,7 +1970,7 @@ def read_version(filename, description, numdefine=None, strdefine=None):
 
             if need_str and wl[1] == strdefine:
                 # Take account of embedded spaces.
-                versstr = string.join(wl[2:])[1:-1]
+                versstr = ' '.join(wl[2:])[1:-1]
                 need_str = 0
 
         l = f.readline()
@@ -1987,7 +1983,7 @@ def read_version(filename, description, numdefine=None, strdefine=None):
     return (vers, versstr)
 
 
-def create_content(dict, macros=None):
+def create_content(cdict, macros=None):
     """Convert a dictionary to a string (typically to use as the content to a
     call to create_config_module()).  Dictionary values that are strings are
     quoted.  Dictionary values that are lists are converted to quoted strings.
@@ -1997,7 +1993,7 @@ def create_content(dict, macros=None):
     """
     content = "_pkg_config = {\n"
 
-    keys = dict.keys()
+    keys = list(cdict.keys())
     keys.sort()
 
     # Format it nicely.
@@ -2010,17 +2006,17 @@ def create_content(dict, macros=None):
             width = klen
 
     for k in keys:
-        val = dict[k]
+        val = cdict[k]
         vtype = type(val)
         delim = None
 
         if val is None:
             val = "None"
-        elif vtype == types.ListType:
-            val = string.join(val)
+        elif vtype == list:
+            val = ' '.join(val)
             delim = "'"
-        elif vtype == types.IntType:
-            if string.find(k, "version") >= 0:
+        elif vtype == int:
+            if k.find("version") >= 0:
                 # Assume it's a hexadecimal version number.  It doesn't matter
                 # if it isn't, we are just trying to make it look pretty.
                 val = "0x%06x" % val
@@ -2036,7 +2032,7 @@ def create_content(dict, macros=None):
 
             val = delim + val + delim
 
-        content = content + "    '" + k + "':" + (" " * (width - len(k) + 2)) + string.replace(val, "\\", "\\\\")
+        content = content + "    '" + k + "':" + (" " * (width - len(k) + 2)) + val.replace("\\", "\\\\")
 
         if k != keys[-1]:
             content = content + ","
@@ -2051,7 +2047,7 @@ def create_content(dict, macros=None):
     if macros:
         content = content + "{\n"
 
-        names = macros.keys()
+        names = list(macros.keys())
         names.sort()
 
         width = 0
@@ -2073,7 +2069,7 @@ def create_content(dict, macros=None):
                 delim = "'"
 
             k = "'" + c + "':"
-            content = content + "    %-*s  %s%s%s%s\n" % (1 + width + 2, k, delim, string.replace(val, "\\", "\\\\"), delim, sep)
+            content = content + "    %-*s  %s%s%s%s\n" % (1 + width + 2, k, delim, val.replace("\\", "\\\\"), delim, sep)
 
         content = content + "}\n"
     else:
@@ -2094,7 +2090,7 @@ def create_config_module(module, template, content, macros=None):
     macros is an optional dictionary of platform specific build macros.  It is
     only used if create_content() is called to convert the content to a string.
     """
-    if type(content) == types.DictType:
+    if type(content) == dict:
         content = create_content(content, macros)
 
     # Allow this file to used as a template.
@@ -2105,7 +2101,7 @@ def create_config_module(module, template, content, macros=None):
 
     line = sf.readline()
     while line:
-        if string.find(line, key) >= 0:
+        if line.find(key) >= 0:
             line = content
 
         df.write(line)
@@ -2126,7 +2122,7 @@ def version_to_sip_tag(version, tags, description):
 
     Returns the corresponding tag.
     """
-    vl = tags.keys()
+    vl = list(tags.keys())
     vl.sort()
 
     # For a snapshot use the latest tag.
@@ -2172,7 +2168,7 @@ def format(msg, leftmargin=0, rightmargin=78):
     curs = leftmargin
     fmsg = " " * leftmargin
 
-    for w in string.split(msg):
+    for w in msg.split():
         l = len(w)
         if curs != leftmargin and curs + l > rightmargin:
             fmsg = fmsg + "\n" + (" " * leftmargin)
@@ -2208,13 +2204,13 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
     if overrides is not None:
         for oride in overrides:
             prefix = ""
-            name_end = string.find(oride, "+=")
+            name_end = oride.find("+=")
 
             if name_end >= 0:
                 prefix = "+"
                 val_start = name_end + 2
             else:
-                name_end = string.find(oride, "=")
+                name_end = oride.find("=")
 
                 if name_end >= 0:
                     val_start = name_end + 1
@@ -2242,12 +2238,12 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
         def _openfile(self, filename):
             try:
                 f = open(filename, 'r')
-            except IOError, detail:
+            except IOError:
                 # If this file is conditional then don't raise an error.
                 if self.cond_fname == filename:
                     return
 
-                error("Unable to open %s: %s" % (filename, detail))
+                error("Unable to open %s" % filename)
 
             if self.currentfile:
                 self.filestack.append(self.currentfile)
@@ -2334,20 +2330,20 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
             else:
                 break
 
-        line = string.strip(line)
+        line = line.strip()
 
         # Ignore comments.
         if line and line[0] != "#":
-            assstart = string.find(line, "+")
+            assstart = line.find("+")
             if assstart > 0 and line[assstart + 1] == '=':
                 assend = assstart + 1
             else:
-                assstart = string.find(line, "=")
+                assstart = line.find("=")
                 assend = assstart
 
             if assstart > 0:
-                lhs = string.strip(line[:assstart])
-                rhs = string.strip(line[assend + 1:])
+                lhs = line[:assstart].strip()
+                rhs = line[assend + 1:].strip()
 
                 # Remove the escapes for any quotes.
                 rhs = rhs.replace(r'\"', '"').replace(r"\'", "'")
@@ -2366,9 +2362,9 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
     macro_prefix = "QMAKE_"
 
-    for lhs in raw.keys():
+    for lhs in list(raw.keys()):
         # Strip any prefix.
-        if string.find(lhs, macro_prefix) == 0:
+        if lhs.find(macro_prefix) == 0:
             reflhs = lhs[len(macro_prefix):]
         else:
             reflhs = lhs
@@ -2380,8 +2376,8 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
         rhs = raw[lhs]
 
         # Resolve any references.
-        estart = string.find(rhs, "$$(")
-        mstart = string.find(rhs, "$$")
+        estart = rhs.find("$$(")
+        mstart = rhs.find("$$")
 
         while mstart >= 0 and mstart != estart:
             rstart = mstart + 2
@@ -2404,7 +2400,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 mend = mend + 1
 
             if term == "]":
-                if properties is None or lhs not in properties.keys():
+                if properties is None or lhs not in list(properties.keys()):
                     error("%s: property '%s' is not defined." % (filename, lhs))
 
                 value = properties[lhs]
@@ -2418,14 +2414,14 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                     value = ""
 
             rhs = rhs[:mstart] + value + rhs[mend:]
-            estart = string.find(rhs, "$$(")
-            mstart = string.find(rhs, "$$")
+            estart = rhs.find("$$(")
+            mstart = rhs.find("$$")
 
         # Expand any POSIX style environment variables.
         pleadin = ["$$(", "$("]
 
         for pl in pleadin:
-            estart = string.find(rhs, pl)
+            estart = rhs.find(pl)
 
             if estart >= 0:
                 nstart = estart + len(pl)
@@ -2434,7 +2430,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
             estart = -1
 
         while estart >= 0:
-            eend = string.find(rhs[nstart:], ")")
+            eend = rhs[nstart:].find(")")
 
             if eend < 0:
                 break
@@ -2451,7 +2447,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
             rhs = rhs[:estart] + env + rhs[eend + 1:]
 
             for pl in pleadin:
-                estart = string.find(rhs, pl)
+                estart = rhs.find(pl)
 
                 if estart >= 0:
                     nstart = estart + len(pl)
@@ -2460,10 +2456,10 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 estart = -1
 
         # Expand any Windows style environment variables.
-        estart = string.find(rhs, "%")
+        estart = rhs.find("%")
 
         while estart >= 0:
-            eend = string.find(rhs[estart + 1:], "%")
+            eend = rhs[estart + 1:].find("%")
 
             if eend < 0:
                 break
@@ -2479,16 +2475,16 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
             rhs = rhs[:estart] + env + rhs[eend + 1:]
 
-            estart = string.find(rhs, "%")
+            estart = rhs.find("%")
 
         refined[reflhs] = rhs
 
     # Handle the user overrides.
-    for lhs in orides.keys():
+    for lhs in list(orides.keys()):
         rhs = refined[lhs]
         oride = orides[lhs]
 
-        if string.find(oride, "+") == 0:
+        if oride.find("+") == 0:
             if rhs:
                 rhs = rhs + " " + oride[1:]
             else:
